#!/usr/bin/env python

"""Tests for `gitlab_usermgmt` package."""

from gitlab_usermgmt import gitlab_usermgmt


def test_update_user_dict_entry_init():
    user_dict_entry = dict(
        username="hanz",
        state="active",
        access={2587: {'access_level': 50,
                       'expires_at': 'never',
                       'web_url': 'https://group'}}
    )
    assert user_dict_entry == gitlab_usermgmt.update_user_dict_entry(
        None, "hanz", "active", 50, "never",
        2587, "https://group")


def test_update_user_dict_entry_update():
    user_dict_entry1 = dict(
        username="hanz",
        state="active",
        access={2587: {'access_level': 50,
                       'expires_at': 'never',
                       'web_url': 'https://group'}}
    )
    user_dict_entry2 = dict(
        username="hanz",
        state="active",
        access={2587: {'access_level': 50,
                       'expires_at': 'never',
                       'web_url': 'https://group'},
                1337: {'access_level': 40,
                       'expires_at': 'never',
                       'web_url': 'https://group2'}}
    )
    user_dict_entry_update1 = gitlab_usermgmt.update_user_dict_entry(
        None, "hanz", "active", 50, "never", 2587, "https://group")
    assert user_dict_entry_update1 == user_dict_entry1
    user_dict_entry_update2 = gitlab_usermgmt.update_user_dict_entry(
        user_dict_entry_update1, "hanz", "active",
        40, "never", 1337, "https://group2")
    assert user_dict_entry_update2 == user_dict_entry2


def test_read_json_rw():
    user_access_dict = dict(
        usergroups={
            'everyone': {
                '17140737': {
                    'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                    'access_level': 30
                }
            },
            'super_owners': {
                '7266413': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt',
                    'access_level': 50
                }
            },
            'devs_munich': {
                '7266433': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-munich',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'devs_berlin': {
                '7266419': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'ops': {
                '7266460': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                    'access_level': 30
                }
            }
        },
        users={
            '408861': {
                'username': 'ayeks',
                'usergroups': [
                    'super_owners'
                ]
            },
            '5506917': {
                'username': 'maik_superdev',
                'usergroups': [
                    'devs_munich',
                    'devs_berlin',
                    'ops'
                ]
            },
            '5506984': {
                'username': 'tom_berlindev',
                'usergroups': [
                    'devs_berlin'
                ]
            },
            '5506951': {
                'username': 'schorsch_munichdev',
                'usergroups': [
                    'devs_munich'
                ]
            },
            '5507013': {
                'username': 'tina_opswoman',
                'usergroups': [
                    'ops'
                ]
            }
        }
    )
    gitlab_usermgmt.write_dict_to_json(user_access_dict, "./tests/tmp-access_definition_test.json")
    user_access_dict_read = gitlab_usermgmt.read_json_to_dict("./tests/tmp-access_definition_test.json")
    assert user_access_dict == user_access_dict_read


def test_read_json_r():
    user_access_dict = dict(
        usergroups={
            'everyone': {
                '17140737': {
                    'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                    'access_level': 30
                }
            },
            'super_owners': {
                '7266413': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt',
                    'access_level': 50
                }
            },
            'devs_munich': {
                '7266433': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-munich',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'devs_berlin': {
                '7266419': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'ops': {
                '7266460': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                    'access_level': 30
                }
            }
        },
        users={
            '5506917': {
                'username': 'maik_superdev',
                'usergroups': [
                    'everyone',
                    'super_owners',
                    'devs_munich',
                    'devs_berlin',
                    'ops'
                ]
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "usergroups": [
                    "everyone",
                    "devs_munich"
                ]
            },
            '5507013': {
                'username': 'tina_opswoman',
                'usergroups': [
                    'devs_munich',
                    'everyone',
                    'ops'
                ]
            },
            '1337': {
                'username': 'brand_new_kid',
                'usergroups': [
                    'everyone',
                    'ops'
                ]
            }
        }
    )
    user_access_dict_read = gitlab_usermgmt.read_json_to_dict("tests/unit/access_definition.json")
    assert user_access_dict == user_access_dict_read


def test_compare_user_access_equal():
    """
    The access_definiton matches the permissions from the actual users dict.
    Nothing should come up in the to_be_* modified dicts.

    :return:
    """
    access_definition = dict(
        usergroups={
            'everyone': {
                '17140737': {
                    'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                    'access_level': 30
                }
            },
            'super_owners': {
                '7266413': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt',
                    'access_level': 50
                }
            },
            'devs_munich': {
                '7266433': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-munich',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'devs_berlin': {
                '7266419': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'ops': {
                '7266460': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                    'access_level': 30
                }
            }
        },
        users={
            '5506917': {
                'username': 'maik_superdev',
                'usergroups': [
                    'everyone',
                    'super_owners',
                    'devs_munich',
                    'devs_berlin',
                    'ops'
                ]
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "usergroups": [
                    "everyone",
                    "devs_munich"
                ]
            },
            '5507013': {
                'username': 'tina_opswoman',
                'usergroups': [
                    'everyone',
                    'ops'
                ]
            }
        }
    )
    users = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                }
            }
        }
    )
    users_result = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {}
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {}
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {}
            }
        }
    )
    users_compared = gitlab_usermgmt.compare_user_access(users, access_definition)
    for user_id, user_info in users.items():
        assert users_compared[str(user_id)] == users_result[str(user_id)]


def test_compare_user_access_to_be_changed():
    """
    The access levels got changed, so they should show up in the to_be_changed dict.
    The highest access_level of any group defines the outcome in the to_be_changed dict.

    Changed access_level definition from 50 to 30 in groups/gitlab_usermgmt for usergroup "super_owners"
    Changed access_level definition from 30 to 50 of "groups/gitlab_usermgmt/apps-germany" for usergroup "devs_munich"

    :return:
    """
    access_definition = dict(
        usergroups={
            'everyone': {
                '17140737': {
                    'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                    'access_level': 30
                }
            },
            'super_owners': {
                '7266413': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt',
                    'access_level': 30
                }
            },
            'devs_munich': {
                '7266433': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-munich',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 50
                }
            },
            'devs_berlin': {
                '7266419': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'ops': {
                '7266460': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                    'access_level': 30
                }
            }
        },
        users={
            '5506917': {
                'username': 'maik_superdev',
                'usergroups': [
                    'everyone',
                    'super_owners',
                    'devs_munich',
                    'devs_berlin',
                    'ops'
                ]
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "usergroups": [
                    "everyone",
                    "devs_munich"
                ]
            },
            '5507013': {
                'username': 'tina_opswoman',
                'usergroups': [
                    'everyone',
                    'ops'
                ]
            }
        }
    )
    users = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                }
            }
        }
    )
    users_result = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 50,
                        "expires_at": None
                    }
                },
                "to_be_granted": {}
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 50,
                        "expires_at": None
                    }
                },
                "to_be_granted": {}
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {}
            }
        }
    )
    users_compared = gitlab_usermgmt.compare_user_access(users, access_definition)
    for user_id, user_info in users.items():
        assert users_compared[str(user_id)] == users_result[str(user_id)]


def test_compare_user_access_to_be_removed():
    """
    The access levels got changes, so they should show up in the to_be_changed dict
    Removed access to "groups/gitlab_usermgmt/apps-germany" from "devs_munich" definition
    Removed usergroup "ops" and "everyone" from "tina_opswoman" definition

    :return:
    """
    access_definition = None
    access_definition = dict(
        usergroups={
            'everyone': {
                '17140737': {
                    'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                    'access_level': 30
                }
            },
            'super_owners': {
                '7266413': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt',
                    'access_level': 50
                }
            },
            'devs_munich': {
                '7266433': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-munich',
                    'access_level': 30
                }
            },
            'devs_berlin': {
                '7266419': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'ops': {
                '7266460': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                    'access_level': 30
                }
            }
        },
        users={
            '5506917': {
                'username': 'maik_superdev',
                'usergroups': [
                    'everyone',
                    'super_owners',
                    'devs_munich',
                    'devs_berlin',
                    'ops'
                ]
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "usergroups": [
                    "everyone",
                    "devs_munich"
                ]
            },
            '5507013': {
                'username': 'tina_opswoman',
                'usergroups': [
                ]
            }
        }
    )
    users = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                }
            }
        }
    )
    users_result = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {}
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {
                    '7275276': {
                        'access_level': 30,
                        'expires_at': None,
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany'
                    }
                },
                "to_be_changed": {},
                "to_be_granted": {}
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_removed": {
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_changed": {},
                "to_be_granted": {}
            }
        }
    )
    users_compared = gitlab_usermgmt.compare_user_access(users, access_definition)
    for user_id, user_info in users.items():
        assert users_compared[str(user_id)] == users_result[str(user_id)]


def test_compare_user_access_to_be_granted():
    """
    This test grants additional access to users.
    Added complete new user with id "1337" with usergroups "everyone" and "ops"
    Removed "docs" access from "maik_superdev", should be granted
    Added "devs_munich" to "tina_opswoman", should grant two access rights

    :return:
    """
    access_definition = dict(
        usergroups={
            'everyone': {
                '17140737': {
                    'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                    'access_level': 30
                }
            },
            'super_owners': {
                '7266413': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt',
                    'access_level': 50
                }
            },
            'devs_munich': {
                '7266433': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-munich',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'devs_berlin': {
                '7266419': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                    'access_level': 30
                },
                '7275276': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                    'access_level': 30
                }
            },
            'ops': {
                '7266460': {
                    'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                    'access_level': 30
                }
            }
        },
        users={
            '5506917': {
                'username': 'maik_superdev',
                'usergroups': [
                    'everyone',
                    'super_owners',
                    'devs_munich',
                    'devs_berlin',
                    'ops'
                ]
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "usergroups": [
                    "everyone",
                    "devs_munich"
                ]
            },
            '5507013': {
                'username': 'tina_opswoman',
                'usergroups': [
                    'devs_munich',
                    'everyone',
                    'ops'
                ]
            },
            '1337': {
                'username': 'brand_new_kid',
                'usergroups': [
                    'everyone',
                    'ops'
                ]
            }
        }
    )
    users = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                }
            }
        }
    )
    users_result = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    '17140737': {
                        'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                        'access_level': 30
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {}
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    '7266433': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-munich',
                        'access_level': 30
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30
                    }
                }
            },
            "1337": {
                "username": "brand_new_kid",
                "access": {},
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                    }
                }
            }
        }
    )
    users_compared = gitlab_usermgmt.compare_user_access(users, access_definition)
    for user_id, user_info in users.items():
        assert users_compared[str(user_id)] == users_result[str(user_id)]


def test_write_user_compared_list():
    users_result = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    '17140737': {
                        'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                        'access_level': 30
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {}
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    '7266433': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-munich',
                        'access_level': 30
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30
                    }
                }
            },
            "1337": {
                "username": "brand_new_kid",
                "access": {},
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                    }
                }
            }
        }
    )
    gitlab_usermgmt.write_user_compared_list("tests/unit/users.json",
                                             "tests/unit/access_definition.json",
                                             "tests/tmp-users_compared_test.json")
    users_compared = gitlab_usermgmt.read_json_to_dict("tests/tmp-users_compared_test.json")
    for user_id, user_info in users_compared.items():
        assert users_compared[str(user_id)] == users_result[str(user_id)]


def test_update_api_input_members_entry():
    input_entry = None
    update_entry_result = dict(
        {
            "members": {
                "create": [[1337, 50, None], [135, 30, "2020-03-25"]],
                "change": [[254, 30, None]],
                "remove": [[123, 30, None]]
            }
        }
    )
    updated_entry = gitlab_usermgmt.update_api_input_members_entry(input_entry, "create", 1337, 50)
    updated_entry = gitlab_usermgmt.update_api_input_members_entry(updated_entry, "create", 135, 30, "2020-03-25")
    updated_entry = gitlab_usermgmt.update_api_input_members_entry(updated_entry, "change", 254, 30)
    updated_entry = gitlab_usermgmt.update_api_input_members_entry(updated_entry, "remove", 123, 30)
    assert updated_entry == update_entry_result


def test_generate_api_input_from_users_compared():
    users_compared = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    }
                },
                "to_be_changed": {
                    '7266460': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                        'access_level': 50,
                        "expires_at": None
                    }
                },
                "to_be_granted": {
                    '17140737': {
                        'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                        'access_level': 30
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": "2024-03-20"
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 50,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_granted": {}
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_removed": {
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_changed": {},
                "to_be_granted": {}
            },
            "1337": {
                "username": "brand_new_kid",
                "access": {},
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                    }
                }
            }
        }
    )

    api_input_result = dict(
        {
            "groups": {
                "7275276": {
                    "members": {
                        "create": [],
                        "change": [
                            ['5506951', 50, '2024-03-20']
                        ],
                        "remove": [
                            ['5506917', 30, None]
                        ]
                    }
                },
                "7266460": {
                    "members": {
                        "create": [
                            ['1337', 30, None]
                        ],
                        "change": [
                            ['5506917', 50, None]
                        ],
                        "remove": [
                            ['5507013', 30, None]
                        ]
                    }
                }

            },
            "projects": {
                "17140737": {
                    "members": {
                        "create": [
                            ['5506917', 30, None],
                            ['1337', 30, None]
                        ],
                        "change": [],
                        "remove": [
                            ['5507013', 30, "2024-03-20"]
                        ]
                    }
                }
            }
        }
    )
    api_input = gitlab_usermgmt.generate_api_input_from_users_compared(users_compared)
    assert api_input == api_input_result



def test_manage_admins_only():
    """
    This test checks how the manage_admins_only function works.
    Deletes all entries in the to_be_removed list. Put them on the
    to_be_changed list if they have no expire date. Sets expire date
    to 1 year in future.

    :return:
    """
    users_compared_input = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    }
                },
                "to_be_changed": {
                    '7266460': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                        'access_level': 50,
                        "expires_at": None
                    }
                },
                "to_be_granted": {
                    '17140737': {
                        'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                        'access_level': 30,
                        "expires_at": None
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 50,
                        "expires_at": None
                    }
                },
                "to_be_granted": {}
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_removed": {
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    },
                    "17140743": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs/xyz",
                        "access_level": 30,
                        "expires_at": "2666-03-20"
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_changed": {},
                "to_be_granted": {}
            },
            "1337": {
                "username": "brand_new_kid",
                "access": {},
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            }
        }
    )
    users_compared_result = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {
                },
                "to_be_changed": {
                    '7266460': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                        'access_level': 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        'expires_at': "2021-03-28"
                    }
                },
                "to_be_granted": {
                    '17140737': {
                        'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                        'access_level': 30,
                        "expires_at": None
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 50,
                        "expires_at": None
                    }
                },
                "to_be_granted": {}
            },
            "5507013": {
                "username": "tina_opswoman",
                "state": "active",
                "access": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_removed": {
                },
                "to_be_changed": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": "2024-03-20"
                    },
                    "17140743": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs/xyz",
                        "access_level": 30,
                        "expires_at": "2024-09-28"
                    }
                },
                "to_be_granted": {}
            },
            "1337": {
                "username": "brand_new_kid",
                "access": {},
                "to_be_removed": {},
                "to_be_changed": {},
                "to_be_granted": {
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                }
            }
        }
    )
    users_compared_output = gitlab_usermgmt.manage_admins_only(users_compared_input, "2024-09-28")
    for user_id, user_info in users_compared_output.items():
        assert users_compared_result[str(user_id)] == users_compared_result[str(user_id)]



def test_remove_expire_dates():
    """
    This test checks how the expire date gets set to None if a date is set.
    Checks only access and to_be_changed list

    :return:
    """
    users_compared_input = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": "2012-03-20"
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    }
                },
                "to_be_changed": {
                    '7266460': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                        'access_level': 50,
                        "expires_at": "2024-03-20"
                    }
                },
                "to_be_granted": {
                    '17140737': {
                        'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                        'access_level': 30
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 50,
                        "expires_at": None
                    }
                },
                "to_be_granted": {}
            }
        }
    )
    users_compared_result = dict(
        {
            "5506917": {
                "username": "maik_superdev",
                "state": "active",
                "access": {
                    "7266413": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt",
                        "access_level": 50,
                        "expires_at": None
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": "2012-03-20"
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": "2012-03-20"
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                    "7266460": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/platform-services",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": "2012-03-20"
                    }
                },
                "to_be_changed": {
                    '7266460': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/platform-services',
                        'access_level': 50,
                        "expires_at": None
                    },
                    '7266419': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-berlin',
                        'access_level': 30,
                        "expires_at": None
                    }
                },
                "to_be_granted": {
                    '17140737': {
                        'web_url': 'https://gitlab.com/gitlab_usermgmt/docs',
                        'access_level': 30
                    }
                }
            },
            "5506951": {
                "username": "schorsch_munichdev",
                "state": "active",
                "access": {
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": "2013-03-20"
                    },
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 30,
                        "expires_at": None
                    },
                    "17140737": {
                        "web_url": "https://gitlab.com/gitlab_usermgmt/docs",
                        "access_level": 30,
                        "expires_at": None
                    }
                },
                "to_be_removed": {},
                "to_be_changed": {
                    '7275276': {
                        'web_url': 'https://gitlab.com/groups/gitlab_usermgmt/apps-germany',
                        'access_level': 50,
                        "expires_at": None
                    },
                    "7266433": {
                        "web_url": "https://gitlab.com/groups/gitlab_usermgmt/apps-munich",
                        "access_level": 30,
                        "expires_at": None
                    },
                },
                "to_be_granted": {}
            }
        }
    )
    users_compared_output = gitlab_usermgmt.remove_expire_dates(users_compared_input)
    for user_id, user_info in users_compared_output.items():
        assert users_compared_result[str(user_id)] == users_compared_result[str(user_id)]
